<?php

namespace Argus\Steam;


class Authorization {


    static function runAuthorizationRedirect () {
        $openid = new LightOpenID($_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['SERVER_NAME']);
        if (!$openid->mode) {
            $openid->identity = 'https://steamcommunity.com/openid';
            header('Location: ' . $openid->authUrl());
            exit;
        }
        return true;
    }


    static function authorizeOrRegisterUser ($redirectTo = 'argusLastPage') {
        self::runAuthorizationRedirect();

        if ($redirectTo == 'argusPrevPage') {
            $redirectURL = $_COOKIE['argusPrevPage'] ?? '/';
        } else {
            $redirectURL = $_COOKIE['argusLastPage'] ?? '/';
        }

        // Steam API did not return data
        if (!isset($_GET['openid_identity'])) {
            $message = __("Steam ID was not provided", 'argus-wordpress-plugin');
            include_once ARGUS_STEAM_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // extracting Steam ID
        $openidIdentityURL = explode('/', parse_url($_GET['openid_identity'])['path']);
        $steamID           = end($openidIdentityURL);

        // Looking for attached Steam profiles by ID
        $usersWithIDInMeta = get_users([
            'meta_key'   => 'argusSteamID',
            'meta_value' => $steamID
        ]);

        if (!count($usersWithIDInMeta)) {

            // Error if user registration are off
            if (!get_option('users_can_register')) {
                $message = __("Registration of new users is disabled", 'argus-wordpress-plugin');
                include_once ARGUS_STEAM_PLUGIN_PATH . 'views/authorization-error.phtml';
                exit();
            }

            $name  = 'steam' . $steamID;
            $email = 'fake.email.' . $steamID . '@email.com';

            // Registering new user
            $registerResult = register_new_user(sanitize_user($name, true), $email);
            if (isset($registerResult->errors)) {
                $message = Data::unwrapArrayInString($registerResult->errors);
                include_once ARGUS_STEAM_PLUGIN_PATH . 'views/authorization-error.phtml';
                exit();
            }
            update_user_meta($registerResult, 'argusSteamID', $steamID);
            $usersWithIDInMeta[0]->ID = $registerResult;
        }

        // Authorizing user
        wp_set_auth_cookie($usersWithIDInMeta[0]->ID, true);

        // Defining final redirect page location
        if ($redirectTo == 'argusLastPage' || $redirectTo == 'argusPrevPage') {
            $finalRedirectURL = $redirectURL;
        } else {
            $redirectArray = json_decode(get_option('argusSteamCustomRedirects'));
            preg_match('/^argus-auth-steam-go-([0-9]+)-page$/', $redirectTo, $parsedRedirectState);
            if (isset($redirectArray->{"$parsedRedirectState[1]"}) && isset($redirectArray->{"$parsedRedirectState[1]"}->url) && $redirectArray->{"$parsedRedirectState[1]"}->url) {
                $finalRedirectURL = $redirectArray->{"$parsedRedirectState[1]"}->url;
            } else {
                $finalRedirectURL = '/';
            }
        }

        wp_redirect($finalRedirectURL);
        exit;
    }


    static function connectProfileToUser () {
        self::runAuthorizationRedirect();

        $redirectURL = $_COOKIE['argusLastPage'] ?? '/';

        // Steam API did not return data
        if (!isset($_GET['openid_identity'])) {
            $message = __("Steam ID was not provided", 'argus-wordpress-plugin');
            include_once ARGUS_STEAM_PLUGIN_PATH . 'views/authorization-error.phtml';
            exit();
        }

        // Getting basic user data
        $openidIdentityURL = explode('/', parse_url($_GET['openid_identity'])['path']);
        $steamID           = end($openidIdentityURL);
        $wpUserID          = get_current_user_id();

        // Processing profile connection
        if ($wpUserID) update_user_meta($wpUserID, 'argusSteamID', $steamID);

        wp_redirect($redirectURL);
        exit();
    }


    static function modifyWPLoginForm () {
        if (get_option('argusSteamModifyWPLoginForm')) {
            $link    = Data::getAuthorizationLink('argus-auth-steam-go-last-page');
            $styling = 'button button-primary button-large';
            include ARGUS_STEAM_PLUGIN_PATH . 'views/includes/login-form.phtml';
        }
    }


    static function addCommentWithSteamButton () {
        if (!is_user_logged_in()) {
            $cpt = explode(',', get_option('argusSteamCommentsCPT'));
            if (!count($cpt)) return;

            $style = '';
            if (get_option('argusSteamStylingOAuthButtons')) {
                $style = 'default';
                wp_enqueue_style('argus-steam-front-end-styles');
            }

            if (in_array(get_post()->post_type, $cpt)) {
                echo do_shortcode('[argus-steam-authorization-button style="' . $style . '"]');
            }
        }
    }

}
